'''
A resolver that searches a list of include paths for YAML files matching the
given name. The include paths must be set prior to validation.
'''

import functools
import pkg_resources
import os

import yaml


DEFAULT_HANDLER_NAME = 'sschema'


def _handler(name, include_paths, uri):
    '''Handles an URI of form {name}://'''
    scheme = '{}://'.format(name)
    assert(uri.startswith(scheme))
    uri = uri[len(scheme):]

    for base in include_paths:
        path = os.path.join(base, uri)
        try:
            f = open(path, 'r')
        except FileNotFoundError:
            continue
        else:
            with f:
                return yaml.safe_load(f)
    raise FileNotFoundError('{} handler could not find path "{}" in any of {}'.format(name, uri, include_paths))


def _make_handler(include_paths, name):
    '''Returns an include handler and name (for use with make_resolver) that
    uses the give include paths.'''
    return (functools.partial(_handler, name, include_paths), name)


def make_default_handler():
    '''Returns a default handler, which uses the common schemas defined in
    sschema as the include path.'''
    schema_path = pkg_resources.resource_filename('sschema', 'schema')
    return _make_handler([schema_path], DEFAULT_HANDLER_NAME)


def make_handler(include_paths, name):
    '''Returns an include handler and name (for use with make_resolver) that
    uses the given include paths.'''
    if name == DEFAULT_HANDLER_NAME:
        raise ValueError('Handler name "{}" is reserved for internal use'.format(name))
    return _make_handler(include_paths, name)



'''
This is the top-level module for sschema.
'''

import types

import jsonschema


class SSchemaError(Exception):
    def __init__(self, msg):
        super().__init__(msg)


def make_format_checker(modules):
    '''Makes a format checker for use in jsonschema validation. Takes a list of
    modules, each of which must have a top-level "check" (which checks a
    format) and a constant "FORMAT_NAME" (the name of the format to check).
    From this list of modules, we create and return a format checker object.'''
    format_checker = jsonschema.FormatChecker()
    for module in modules:
        func = module.check
        name = module.FORMAT_NAME
        # Manually decorate the format checker with our format.
        format_checker.checks(name)(func)
    return format_checker


def make_resolver(schema, items):
    '''Makes a resolver for use in jsonschema validation. Takes a schema (a
    required argument of RefResolver.__init__ and either:

    - a list of modules, each of which has a top-level "handler" (a handler
      function) and a constant "HANDLER_NAME" (the name of the handler prefix).
    - a list of tuples of the form (function, name), where function is a
      handler function and name is a handler prefix.

    From this we create and return a RefResolver object.'''
    handlers = {}
    for item in items:
        if isinstance(item, types.ModuleType):
            func = item.handler
            name = item.HANDLER_NAME
        elif isinstance(item, tuple) or isinstance(item, list):
            if len(item) != 2:
                raise SSchemaError(
                    "handler tuple/list %s doesn't have length 2" % item)
            func, name = item
        else:
            raise SSchemaError(
                'item %s is not a module or a handler tuple/list' % item)
        handlers[name] = func
    resolver = jsonschema.RefResolver.from_schema(schema, handlers=handlers)
    return resolver



#!/usr/bin/python3

import os
from setuptools import setup
from setuptools.command.install import install
import stat


class FixPermsInstall(install):
    '''Custom install class just to fix permissions for the schema data.'''
    def run(self):
        '''Fix schema data permissions.'''
        install.run(self)

        no_exec_mask = ~(stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
        for filepath in self.get_outputs():
            parent = os.path.basename(os.path.dirname(filepath))
            if parent != 'schema':
                continue

            # Open the file so there's no race between the stat and the
            # chmod.
            with open(filepath, 'r') as f:
                fd = f.fileno()
                st = os.fstat(fd)
                os.fchmod(fd, st.st_mode & no_exec_mask)

# Keep the long description in sync with the README.
script_dir = os.path.dirname(os.path.realpath(__file__))
readme_path = os.path.join(script_dir, 'README.md')
with open(readme_path, 'r') as f:
    long_desc = f.read()

setup(
    name='sschema',
    version='0.1.3',
    description='A library and prebuilt schemas for handling edge sensor data',
    long_description=long_desc,
    long_description_content_type='text/markdown',
    author='Martin Kelly',
    author_email='mkelly@xevo.com',
    license='Apache',
    python_requires='>=3',
    install_requires=[
        'PyYAML',
        'simpleeval',
        'jsonschema'
    ],
    packages=['sschema', 'sschema.handler'],
    package_data={'sschema': ['schema/*']},
    cmdclass={
        'install': FixPermsInstall
    },
    classifiers=['Development Status :: 3 - Alpha',
                 'Environment :: Console',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Natural Language :: English',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python :: 3 :: Only',
                 'Topic :: Software Development :: Embedded Systems',
                 'Topic :: Software Development :: Testing :: Acceptance',
                 ]
)
